"""
Django admin declarations
"""

# Django
from django.contrib import admin, messages

# AA SRP
from aasrp.models import AaSrpLink, AaSrpRequest, AaSrpRequestComment, FleetType


def custom_filter(title):
    """
    Custom filter for model properties
    :param title:
    :return:
    """

    class Wrapper(admin.FieldListFilter):
        """
        Custom_filter :: wrapper
        """

        def expected_parameters(self):
            """
            Expected parameters
            :return:
            :rtype:
            """

            pass

        def choices(self, changelist):
            """
            Choices
            :param changelist:
            :type changelist:
            :return:
            :rtype:
            """

            pass

        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


@admin.register(AaSrpLink)
class AaSrpLinkAdmin(admin.ModelAdmin):
    """
    AaSrpLinkAdmin
    """

    list_display = (
        "srp_code",
        "fleet_time",
        "_creator",
        "srp_name",
        "srp_status",
        "fleet_doctrine",
    )
    ordering = ("fleet_time",)

    list_filter = ("creator", "srp_status", "fleet_doctrine")

    search_fields = ("srp_code", "fleet_doctrine", "srp_name")

    @classmethod
    @admin.display(description="Creator", ordering="creator")
    def _creator(cls, obj):
        creator_name = obj.creator
        if obj.creator.profile.main_character:
            creator_name = obj.creator.profile.main_character.character_name

        return creator_name


@admin.register(AaSrpRequest)
class AaSrpRequestAdmin(admin.ModelAdmin):
    """
    AaSrpRequestAdmin
    """

    list_display = (
        "request_code",
        "_creator",
        "character",
        "srp_link",
        "post_time",
        "ship",
        "loss_amount",
        "payout_amount",
        "killboard_link",
        "request_status",
    )
    ordering = ("post_time",)

    list_filter = ("creator", "character", "request_status")

    search_fields = (
        "request_code",
        "character__character_name",
        "ship__name",
        "srp_link__srp_code",
    )

    @classmethod
    @admin.display(description="Creator", ordering="creator")
    def _creator(cls, obj):
        creator_name = obj.creator
        if obj.creator.profile.main_character:
            creator_name = obj.creator.profile.main_character.character_name

        return creator_name


@admin.register(AaSrpRequestComment)
class AaSrpRequestCommentAdmin(admin.ModelAdmin):
    """
    AaSrpRequestCommentAdmin
    """

    list_display = ("srp_request", "comment_type", "creator")
    ordering = ("srp_request",)
    list_filter = ("comment_type",)


@admin.register(FleetType)
class FleetTypeAdmin(admin.ModelAdmin):
    """
    Config for fleet type model
    """

    list_display = ("id", "_name", "_is_enabled")
    list_filter = ("is_enabled",)
    ordering = ("name",)

    @admin.display(description="Fleet Type", ordering="name")
    def _name(self, obj):
        """
        Rewrite name
        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @admin.display(description="Is Enabled", boolean=True, ordering="is_enabled")
    def _is_enabled(self, obj):
        """
        Rewrite is_enabled
        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.is_enabled

    actions = (
        "activate",
        "deactivate",
    )

    @admin.action(description="Activate selected fleet type(s)")
    def activate(self, request, queryset):
        """
        Mark fleet type as active
        :param request:
        :type request:
        :param queryset:
        :type queryset:
        :return:
        :rtype:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = True
                obj.save()

                notifications_count += 1
            except Exception:
                failed += 1

        if failed:
            messages.error(request, f"Failed to activate {failed} fleet types")

        if queryset.count() - failed > 0:
            messages.success(request, f"Activated {notifications_count} fleet type(s)")

    @admin.action(description="Deactivate selected fleet type(s)")
    def deactivate(self, request, queryset):
        """
        Mark fleet type as inactive
        :param request:
        :type request:
        :param queryset:
        :type queryset:
        :return:
        :rtype:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = False
                obj.save()

                notifications_count += 1
            except Exception:
                failed += 1

        if failed:
            messages.error(request, f"Failed to deactivate {failed} fleet types")

        if queryset.count() - failed > 0:
            messages.success(
                request, f"Deactivated {notifications_count} fleet type(s)"
            )



"""
aasrp url config
"""

# Django
from django.urls import include, path

# AA SRP
from aasrp import views

app_name: str = "aasrp"

urlpatterns = [
    path(route="", view=views.dashboard, name="dashboard"),
    path(
        route="all/", view=views.dashboard, kwargs={"show_all_links": True}, name="all"
    ),
    path(route="add/", view=views.srp_link_add, name="add_srp_link"),
    path(
        route="srp-link/<str:srp_code>/edit/",
        view=views.srp_link_edit,
        name="edit_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/view-srp-requests/",
        view=views.srp_link_view_requests,
        name="view_srp_requests",
    ),
    path(
        route="srp-link/<str:srp_code>/enable/",
        view=views.enable_srp_link,
        name="enable_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/disable/",
        view=views.disable_srp_link,
        name="disable_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/delete/",
        view=views.delete_srp_link,
        name="delete_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/request-srp/",
        view=views.request_srp,
        name="request_srp",
    ),
    path(
        route="srp-link/<str:srp_code>/complete/",
        view=views.complete_srp_link,
        name="complete_srp_link",
    ),
    # Ajax calls
    path(
        route="ajax/",
        view=include(
            [
                path(
                    # Get active SRP links
                    route="active-srp-links-data/",
                    view=views.ajax_dashboard_srp_links_data,
                    name="ajax_dashboard_srp_links_data",
                ),
                path(
                    # Get all SRP links
                    route="active-srp-links-data/all/",
                    view=views.ajax_dashboard_srp_links_data,
                    kwargs={"show_all_links": True},
                    name="ajax_dashboard_srp_links_all_data",
                ),
                path(
                    # Get all SRP request for the current user
                    route="user-srp-requests-data/",
                    view=views.ajax_dashboard_user_srp_requests_data,
                    name="ajax_dashboard_user_srp_requests_data",
                ),
                path(
                    # Get all SRP requests for the current SRP link
                    route="srp-link/<str:srp_code>/view-srp-requests-data/",
                    view=views.ajax_srp_link_view_requests_data,
                    name="ajax_srp_link_view_requests_data",
                ),
                path(
                    # Get addition information for the current SRP request
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/view-additional-information-data/"
                    ),
                    view=views.ajax_srp_request_additional_information,
                    name="ajax_srp_request_additional_information",
                ),
                path(
                    # Change the SRP payout amount
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/change-srp-payout/"
                    ),
                    view=views.ajax_srp_request_change_payout,
                    name="ajax_srp_request_change_payout",
                ),
                path(
                    # Change the SRP payout amount
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/approve/"
                    ),
                    view=views.ajax_srp_request_approve,
                    name="ajax_srp_request_approve",
                ),
                path(
                    # Change the SRP payout amount
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/deny/"
                    ),
                    view=views.ajax_srp_request_deny,
                    name="ajax_srp_request_deny",
                ),
                path(
                    # Change the SRP payout amount
                    route=(
                        "srp-link/<str:srp_code>/srp-request/"
                        "<str:srp_request_code>/remove/"
                    ),
                    view=views.ajax_srp_request_remove,
                    name="ajax_srp_request_remove",
                ),
            ]
        ),
    ),
]



"""
User helper
"""

# Django
from django.contrib.auth.models import User

# AA SRP
from aasrp.models import AaSrpUserSettings


def get_user_settings(user: User) -> AaSrpUserSettings:
    """
    Get a users settings or create them
    :param user:
    :return:
    """

    user_settings, _ = AaSrpUserSettings.objects.get_or_create(user=user)

    return user_settings



"""
App settings
"""

# Django
from django.apps import apps

# AA SRP
from aasrp.utils import clean_setting

# SRP Team Discord Channel
AASRP_SRP_TEAM_DISCORD_CHANNEL = clean_setting(
    name="AASRP_SRP_TEAM_DISCORD_CHANNEL", default_value=None, required_type=int
)


def allianceauth_discordbot_installed() -> bool:
    """
    Check if allianceauth-discordbot is installed and active
    :return:
    """

    return apps.is_installed("aadiscordbot")


def aa_discordnotify_installed() -> bool:
    """
    Check if allianceauth-discordbot is installed and active
    :return:
    """

    return apps.is_installed("discordnotify")


def discordproxy_installed() -> bool:
    """
    Check if discordproxy is installed by trying to import the DiscordClient
    :return:
    """

    try:
        # Third Party
        from discordproxy.client import DiscordClient  # noqa: F401
    except ModuleNotFoundError:
        return False
    else:
        return True
